@@ -1,6 +1,7 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1999-2001, 2007, 2009-2014 Free Software Foundation, Inc.
+    Copyright (C) 1999-2001, 2007, 2009-2014, 2019 Free Software Foundation,
+    Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -45,8 +46,8 @@ extern "C" {
 
 extern const char *ped_get_version () _GL_ATTRIBUTE_CONST;
 
-extern void* ped_malloc (size_t size);
-extern void* ped_calloc (size_t size);
+extern void* __attribute__ ((malloc)) ped_malloc (size_t size);
+extern void* __attribute__ ((malloc)) ped_calloc (size_t size);
 extern void free (void* ptr);
 
 #ifdef __cplusplus
