@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 2000, 2007-2014 Free Software Foundation, Inc.
+    Copyright (C) 2000, 2007-2014, 2019 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -432,7 +432,7 @@ closest (PedSector sector, PedSector a, PedSector b)
 	if (b == -1)
 		return a;
 
-	if (abs (sector - a) < abs (sector - b))
+	if (llabs (sector - a) < llabs (sector - b))
 		return a;
 	else
 		return b;
