@@ -1,6 +1,7 @@
 /*
-    reiserfs.c -- libparted / libreiserfs glue
-    Copyright (C) 2001-2002, 2007, 2009-2014 Free Software Foundation, Inc.
+    reiserfs.c -- ReiserFS detection
+    Copyright (C) 2001-2002, 2007, 2009-2014, 2019 Free Software Foundation,
+    Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,17 +15,6 @@
 
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
-
-    This is all rather complicated.  There are a few combinations:
-	* shared libraries full support
-	* dynamic libraries present full support (via dlopen)
-	* dynamic libraries absent (full support disabled) (via dlopen)
-	* discover only
-
-    We'd love to hear comments...
-
-    So far, we've opted for maximum flexibility for the user.  Is it
-    all worth it?
 */
 
 #include <config.h>
@@ -49,9 +39,6 @@
 static PedSector reiserfs_super_offset[] = { 128, 16, -1 };
 static PedFileSystemType* reiserfs_type;
 
-#define FPTR
-#define FCLASS extern
-
 static PedGeometry *reiserfs_probe(PedGeometry *geom)
 {
 	int i;
