@@ -106,28 +106,22 @@ fdasd_cleanup (fdasd_anchor_t *anchor)
 {
 	PDEBUG
 	int i;
-	partition_info_t *p, *q;
+	partition_info_t *part_info, *next;
 
 	if (anchor == NULL)
 		return;
 
-        free(anchor->f4);
-        free(anchor->f5);
-        free(anchor->f7);
-        free(anchor->f9);
-        free(anchor->vlabel);
+	free(anchor->f4);
+	free(anchor->f5);
+	free(anchor->f7);
+	free(anchor->vlabel);
 
-	p = anchor->first;
-	if (p == NULL)
-		return;
-
-	for (i=1; i <= USABLE_PARTITIONS; i++) {
-		if (p == NULL)
-			return;
-		q = p->next;
-		free(p->f1);
-		free(p);
-		p = q;
+	part_info = anchor->first;
+	for (i = 1; i <= USABLE_PARTITIONS && part_info != NULL; i++) {
+		next = part_info->next;
+		free(part_info->f1);
+		free(part_info);
+		part_info = next;
 	}
 }
 
@@ -190,6 +184,9 @@ fdasd_error (fdasd_anchor_t *anc, enum fdasd_failure why, char const *str)
 				_("Device verification failed"),
 				_("The specified device is not a valid DASD device"));
 			break;
+		case volser_not_found:
+			sprintf(error, "fdasd: %s -- %s\n", _("VOLSER not found on device"), str);
+			break;
 		default:
 			sprintf(error, "fdasd: %s: %s\n", _("Fatal error"), str);
 	}
@@ -210,27 +207,7 @@ fdasd_initialize_anchor (fdasd_anchor_t * anc)
 	partition_info_t *p = NULL;
 	partition_info_t *q = NULL;
 
-	anc->devno             = 0;
-	anc->dev_type          = 0;
-	anc->used_partitions   = 0;
-
-	anc->silent            = 0;
-	anc->verbose           = 0;
-	anc->big_disk          = 0;
-	anc->volid_specified   = 0;
-	anc->config_specified  = 0;
-	anc->auto_partition    = 0;
-	anc->devname_specified = 0;
-	anc->print_table       = 0;
-
-	anc->option_reuse      = 0;
-	anc->option_recreate   = 0;
-
-	anc->vlabel_changed    = 0;
-	anc->vtoc_changed      = 0;
-	anc->blksize           = 0;
-	anc->fspace_trk        = 0;
-	anc->label_pos         = 0;
+	bzero(anc, sizeof(fdasd_anchor_t));
 
 	for (i=0; i<USABLE_PARTITIONS; i++)
 		setpos(anc, i, -1);
@@ -272,24 +249,18 @@ fdasd_initialize_anchor (fdasd_anchor_t * anc)
 		if (p == NULL)
 			fdasd_error(anc, malloc_failed,
 				    _("No room for partition info."));
-		p->used       = 0x00;
-		p->len_trk    = 0;
-		p->start_trk  = 0;
-		p->fspace_trk = 0;
-		p->type       = 0;
+		bzero(p, sizeof(partition_info_t));
 
 		/* add p to double pointered list */
 		if (i == 1) {
-	        anc->first = p;
-			p->prev = NULL;
+		        anc->first = p;
 		} else if (i == USABLE_PARTITIONS) {
-	        anc->last = p;
-	        p->next = NULL;
+		        anc->last = p;
 			p->prev = q;
 			q->next = p;
 		} else {
-	        p->prev = q;
-	        q->next = p;
+	                p->prev = q;
+	                q->next = p;
 		}
 
 		p->f1 = malloc(sizeof(format1_label_t));
@@ -313,7 +284,7 @@ fdasd_write_vtoc_labels (fdasd_anchor_t * anc, int fd)
 	PDEBUG
 	partition_info_t *p;
 	unsigned long b, maxblk;
-	char dsno[6], s1[7], s2[45], *c1, *c2, *ch;
+	char dsno[6], s1[VOLSER_LENGTH + 1], s2[45], *c1, *c2, *ch;
 	int i = 0, k = 0;
 	cchhb_t f9addr;
 	format1_label_t emptyf1;
@@ -545,7 +516,6 @@ fdasd_recreate_vtoc (fdasd_anchor_t *anc)
 	int i;
 
 	vtoc_init_format4_label(anc->f4,
-							USABLE_PARTITIONS,
 							anc->geo.cylinders,
 						anc->formatted_cylinders,
 							anc->geo.heads,
@@ -793,7 +763,7 @@ fdasd_invalid_vtoc_pointer(fdasd_anchor_t *anc)
 	anc->formatted_cylinders = anc->hw_cylinders;
 	anc->fspace_trk = anc->formatted_cylinders * anc->geo.heads
 			- FIRST_USABLE_TRK;
-	vtoc_init_format4_label(anc->f4, USABLE_PARTITIONS,
+	vtoc_init_format4_label(anc->f4,
 			anc->geo.cylinders, anc->formatted_cylinders,
 			anc->geo.heads, anc->geo.sectors,
 			anc->blksize, anc->dev_type);
@@ -807,6 +777,8 @@ fdasd_invalid_vtoc_pointer(fdasd_anchor_t *anc)
 			anc->formatted_cylinders, anc->geo.heads);
 
 	vtoc_set_cchhb(&anc->vlabel->vtoc, VTOC_START_CC, VTOC_START_HH, 0x01);
+	anc->vtoc_changed++;
+	anc->vlabel_changed++;
 }
 
 /*
@@ -818,7 +790,7 @@ fdasd_process_invalid_vtoc(fdasd_anchor_t *anc)
 	anc->formatted_cylinders = anc->hw_cylinders;
 	anc->fspace_trk = anc->formatted_cylinders * anc->geo.heads
 			- FIRST_USABLE_TRK;
-	vtoc_init_format4_label(anc->f4, USABLE_PARTITIONS,
+	vtoc_init_format4_label(anc->f4,
 			anc->geo.cylinders, anc->formatted_cylinders,
 			anc->geo.heads, anc->geo.sectors,
 			anc->blksize, anc->dev_type);
@@ -829,6 +801,8 @@ fdasd_process_invalid_vtoc(fdasd_anchor_t *anc)
 			FIRST_USABLE_TRK,
 			anc->formatted_cylinders * anc->geo.heads - 1,
 			anc->formatted_cylinders, anc->geo.heads);
+
+	anc->vtoc_changed++;
 }
 
 
@@ -901,7 +875,7 @@ fdasd_check_volume (fdasd_anchor_t *anc, int fd)
 
 		fdasd_init_volume_label(anc, fd);
 
-		vtoc_init_format4_label(anc->f4, USABLE_PARTITIONS,
+		vtoc_init_format4_label(anc->f4,
 				anc->geo.cylinders, anc->formatted_cylinders,
 				anc->geo.heads, anc->geo.sectors,
 				anc->blksize, anc->dev_type);
@@ -946,16 +920,78 @@ fdasd_check_api_version (fdasd_anchor_t *anc, int f)
 	}
 }
 
+/*
+ * The following two functions match those in the DASD ECKD device driver.
+ * They are used to compute how many records of a given size can be stored
+ * in one track.
+ */
+static unsigned int ceil_quot(unsigned int d1, unsigned int d2)
+{
+	return (d1 + (d2 - 1)) / d2;
+}
+
+/* kl: key length, dl: data length */
+static unsigned int recs_per_track(unsigned short dev_type, unsigned int kl,
+				   unsigned int dl)
+{
+	unsigned int dn, kn;
+
+	switch (dev_type) {
+	case DASD_3380_TYPE:
+		if (kl)
+			return 1499 / (15 + 7 + ceil_quot(kl + 12, 32) +
+				       ceil_quot(dl + 12, 32));
+		else
+			return 1499 / (15 + ceil_quot(dl + 12, 32));
+	case DASD_3390_TYPE:
+		dn = ceil_quot(dl + 6, 232) + 1;
+		if (kl) {
+			kn = ceil_quot(kl + 6, 232) + 1;
+			return 1729 / (10 + 9 + ceil_quot(kl + 6 * kn, 34) +
+				       9 + ceil_quot(dl + 6 * dn, 34));
+		} else
+			return 1729 / (10 + 9 + ceil_quot(dl + 6 * dn, 34));
+	case DASD_9345_TYPE:
+		dn = ceil_quot(dl + 6, 232) + 1;
+		if (kl) {
+			kn = ceil_quot(kl + 6, 232) + 1;
+			return 1420 / (18 + 7 + ceil_quot(kl + 6 * kn, 34) +
+				       ceil_quot(dl + 6 * dn, 34));
+		} else
+			return 1420 / (18 + 7 + ceil_quot(dl + 6 * dn, 34));
+	}
+	return 0;
+}
+
+/*
+ * Verify that number of tracks (heads) per cylinder and number of
+ * sectors per track match the expected values for a given device type
+ * and block size.
+ * Returns 1 for a valid match and 0 otherwise.
+ */
+static int fdasd_verify_geometry(unsigned short dev_type, int blksize,
+				 struct fdasd_hd_geometry *geometry)
+{
+	unsigned int expected_sectors;
+	if (geometry->heads != 15)
+		return 0;
+	expected_sectors = recs_per_track(dev_type, 0, blksize);
+	if (geometry->sectors == expected_sectors)
+		return 1;
+	return 0;
+}
+
 /*
  * reads dasd geometry data
  */
-void
+int
 fdasd_get_geometry (const PedDevice *dev, fdasd_anchor_t *anc, int f)
 {
 	PDEBUG
 	int blksize = 0;
 	dasd_information_t dasd_info;
 	struct dasd_eckd_characteristics *characteristics;
+	unsigned long long size_in_bytes;
 
 	/* We can't get geometry from a regular file,
 	   so simulate something usable, for the sake of testing.  */
@@ -979,36 +1015,73 @@ fdasd_get_geometry (const PedDevice *dev, fdasd_anchor_t *anc, int f)
 				anc->geo.heads;
 	    anc->is_file = 1;
 	} else {
-		if (ioctl(f, HDIO_GETGEO, &anc->geo) != 0)
+	        if (ioctl(f, BLKGETSIZE64, &size_in_bytes) != 0) {
+		        fdasd_error(anc, unable_to_ioctl,
+				    _("Could not retrieve disk size."));
+			goto error;
+		}
+
+		if (ioctl(f, HDIO_GETGEO, &anc->geo) != 0 ||
+		        anc->geo.heads == 0 ||
+		        anc->geo.sectors == 0 ||
+		        anc->geo.cylinders == 0 ) {
 			fdasd_error(anc, unable_to_ioctl,
 			    _("Could not retrieve disk geometry information."));
+			goto error;
+		}
 
-		if (ioctl(f, BLKSSZGET, &blksize) != 0)
+		if (ioctl(f, BLKSSZGET, &blksize) != 0) {
 			fdasd_error(anc, unable_to_ioctl,
 			    _("Could not retrieve blocksize information."));
+			goto error;
+		}
 
 		/* get disk type */
-		if (ioctl(f, BIODASDINFO, &dasd_info) != 0)
-			fdasd_error(anc, unable_to_ioctl,
-				    _("Could not retrieve disk information."));
-
-		characteristics = (struct dasd_eckd_characteristics *)
-					&dasd_info.characteristics;
-		if (characteristics->no_cyl == LV_COMPAT_CYL &&
-		    characteristics->long_no_cyl)
-			anc->hw_cylinders = characteristics->long_no_cyl;
-		else
-			anc->hw_cylinders = characteristics->no_cyl;
+		if (ioctl(f, BIODASDINFO, &dasd_info) != 0) {
+		        /* verify that the geometry matches a 3390 DASD */
+		        if (!fdasd_verify_geometry(DASD_3390_TYPE, blksize,
+						   &anc->geo)) {
+			        fdasd_error(anc, wrong_disk_type,
+					    _("Disk geometry does not match a " \
+					      "DASD device of type 3390."));
+				goto error;
+			}
+			anc->dev_type = DASD_3390_TYPE;
+			anc->hw_cylinders =
+			        size_in_bytes / (blksize * anc->geo.heads * anc->geo.sectors);
+			/* The VOL1 label on a CDL formatted ECKD DASD is in block 2
+			 * It will be verified later, if this position actually holds a
+			 * valid label record.
+			 */
+			anc->label_pos = 2 * blksize;
+			/* A devno 0 is actually a valid devno, which could exist
+			 * in the system. Since we use this number only to create
+			 * a default volume serial, there is no serious conflict.
+			 */
+			anc->devno = 0;
+		} else {
+		        characteristics = (struct dasd_eckd_characteristics *)
+			        &dasd_info.characteristics;
+			if (characteristics->no_cyl == LV_COMPAT_CYL &&
+			        characteristics->long_no_cyl)
+		                anc->hw_cylinders = characteristics->long_no_cyl;
+			else
+		                anc->hw_cylinders = characteristics->no_cyl;
+			anc->dev_type = dasd_info.dev_type;
+			anc->label_pos = dasd_info.label_block * blksize;
+			anc->devno = dasd_info.devno;
+			anc->label_block = dasd_info.label_block;
+			anc->FBA_layout = dasd_info.FBA_layout;
+		}
 
 		anc->is_file = 0;
 	}
 
-	anc->dev_type   = dasd_info.dev_type;
-	anc->blksize    = blksize;
-	anc->label_pos  = dasd_info.label_block * blksize;
-	anc->devno      = dasd_info.devno;
-	anc->label_block = dasd_info.label_block;
-	anc->FBA_layout = dasd_info.FBA_layout;
+	anc->blksize = blksize;
+	return 1;
+
+ error:
+	return 0;
 }
 
 /*
@@ -1213,12 +1286,10 @@ fdasd_add_partition (fdasd_anchor_t *anc, unsigned int start,
 		return 0;
 
 	if (anc->formatted_cylinders > LV_COMPAT_CYL) {
-		vtoc_init_format8_label(anc->vlabel->volid, anc->blksize, &ext,
-					p->f1);
+		vtoc_init_format8_label(anc->blksize, &ext, p->f1);
 	} else {
 		PDEBUG;
-		vtoc_init_format1_label(anc->vlabel->volid, anc->blksize, &ext,
-					p->f1);
+		vtoc_init_format1_label(anc->blksize, &ext, p->f1);
 	}
 
 	PDEBUG;
@@ -1245,4 +1316,127 @@ fdasd_add_partition (fdasd_anchor_t *anc, unsigned int start,
 	return p;
 }
 
+/*
+ * Check for valid volume serial characters (max. 6) - remove invalid.
+ * If volser is empty, fill with default volser.
+ */
+void fdasd_check_volser (char *volser, int devno)
+{
+	int from, to;
+
+	for (from = 0, to = 0; volser[from] && from < VOLSER_LENGTH; from++) {
+
+			if ((volser[from] >= 0x23 &&
+			     volser[from] <= 0x25) ||
+			    (volser[from] >= 0x30 &&
+			     volser[from] <= 0x39) ||
+			    (volser[from] >= 0x40 &&
+			     volser[from] <= 0x5a) ||
+			    (volser[from] >= 0x61 &&
+			     volser[from] <= 0x7a))
+				volser[to++] = toupper(volser[from]);
+	}
+
+	volser[to] = 0x00;
+
+	if (volser[0] == 0x00)
+		sprintf(volser, "0X%04x", devno);
+}
+
+/*
+ * get volser from vtoc
+ */
+int fdasd_get_volser (fdasd_anchor_t *anc, char *volser, int fd)
+{
+	volume_label_t vlabel;
+
+	vtoc_read_volume_label(fd, anc->label_pos, &vlabel);
+	vtoc_volume_label_get_volser(&vlabel, volser);
+	return 0;
+}
+
+/* Changes the volume serial (menu option)
+ *
+ */
+void fdasd_change_volser (fdasd_anchor_t *anc, char *str)
+{
+	fdasd_check_volser(str, anc->devno);
+	vtoc_volume_label_set_volser(anc->vlabel, str);
+
+	vtoc_set_cchhb(&anc->vlabel->vtoc, VTOC_START_CC, VTOC_START_HH, 0x01);
+	anc->vlabel_changed++;
+	anc->vtoc_changed++;
+}
+
+/*
+ * re-create all VTOC labels, but use the partition information
+ * from existing VTOC
+ */
+void fdasd_reuse_vtoc (fdasd_anchor_t *anc)
+{
+	partition_info_t *part_info = anc->first;
+	struct fdasd_hd_geometry geo = anc->geo;
+	format1_label_t f1;
+	format4_label_t f4;
+	format5_label_t f5;
+	format7_label_t f7;
+
+	vtoc_init_format4_label(&f4, geo.cylinders, anc->formatted_cylinders,
+				geo.heads, geo.sectors,
+				anc->blksize, anc->dev_type);
+
+	/* reuse some FMT4 values */
+	f4.DS4HPCHR = anc->f4->DS4HPCHR;
+	f4.DS4DSREC = anc->f4->DS4DSREC;
+
+	/* re-initialize both free-space labels */
+	vtoc_init_format5_label(&f5);
+	vtoc_init_format7_label(&f7);
+
+	if (anc->fspace_trk > 0)
+		vtoc_set_freespace(&f4, &f5, &f7, '+', anc->verbose,
+				   FIRST_USABLE_TRK,
+				   FIRST_USABLE_TRK + anc->fspace_trk - 1,
+				   anc->formatted_cylinders, geo.heads);
+
+	while (part_info != NULL) {
+		if (part_info->used != 0x01) {
+			part_info = part_info->next;
+			continue;
+		}
+
+		if (anc->formatted_cylinders > LV_COMPAT_CYL)
+			vtoc_init_format8_label(anc->blksize,
+						&part_info->f1->DS1EXT1, &f1);
+		else
+			vtoc_init_format1_label(anc->blksize,
+						&part_info->f1->DS1EXT1, &f1);
+
+
+		strncpy(f1.DS1DSNAM, part_info->f1->DS1DSNAM, 44);
+		strncpy((char *)f1.DS1DSSN, (char *)part_info->f1->DS1DSSN, 6);
+		f1.DS1CREDT = part_info->f1->DS1CREDT;
+
+		memcpy(part_info->f1, &f1, sizeof(format1_label_t));
+
+		if (part_info->fspace_trk > 0)
+			vtoc_set_freespace(&f4, &f5, &f7, '+', anc->verbose,
+					   part_info->end_trk + 1,
+					   part_info->end_trk +
+					   part_info->fspace_trk,
+					   anc->formatted_cylinders, geo.heads);
+
+		part_info = part_info->next;
+	}
+
+	/* over-write old labels with new ones */
+	memcpy(anc->f4, &f4, sizeof(format4_label_t));
+	memcpy(anc->f5, &f5, sizeof(format5_label_t));
+	memcpy(anc->f7, &f7, sizeof(format7_label_t));
+
+	anc->vtoc_changed++;
+
+	return;
+}
+
 /* vim:set tabstop=4 shiftwidth=4 softtabstop=4: */
